"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _msw = require("msw");

var _react = require("@testing-library/react");

var _mocks = require("./helpers/mocks");

var _utils = require("./helpers/utils");

var _react2 = require("../react");

var _jsxRuntime = require("react/jsx-runtime");

var _excluded = ["testId"];

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var origDocumentVisibility = document.visibilityState;
var fetchSpy = jest.spyOn(global, "fetch");
beforeAll(function () {
  _mocks.server.listen();
});
afterEach(function () {
  _mocks.server.resetHandlers();

  changeTabVisibility(origDocumentVisibility);
  fetchSpy.mockClear();
});
afterAll(function () {
  _mocks.server.close();
});
test("fetches the session once and re-uses it for different consumers", (0, _asyncToGenerator2.default)(_regenerator.default.mark(function _callee() {
  return _regenerator.default.wrap(function _callee$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          (0, _react.render)((0, _jsxRuntime.jsx)(ProviderFlow, {}));
          expect(_react.screen.getByTestId("session-1")).toHaveTextContent("loading");
          expect(_react.screen.getByTestId("session-2")).toHaveTextContent("loading");
          return _context.abrupt("return", (0, _react.waitFor)(function () {
            expect(fetchSpy).toHaveBeenCalledTimes(1);
            expect(fetchSpy).toHaveBeenCalledWith("/api/auth/session", expect.anything());

            var session1 = _react.screen.getByTestId("session-1").textContent;

            var session2 = _react.screen.getByTestId("session-2").textContent;

            expect(session1).toEqual(session2);
          }));

        case 4:
        case "end":
          return _context.stop();
      }
    }
  }, _callee);
})));
test("when there's an existing session, it won't try to fetch a new one straightaway", (0, _asyncToGenerator2.default)(_regenerator.default.mark(function _callee2() {
  return _regenerator.default.wrap(function _callee2$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          (0, _react.render)((0, _jsxRuntime.jsx)(ProviderFlow, {
            session: _mocks.mockSession
          }));
          expect(fetchSpy).not.toHaveBeenCalled();

        case 2:
        case "end":
          return _context2.stop();
      }
    }
  }, _callee2);
})));
test("will refetch the session when the browser tab becomes active again", (0, _asyncToGenerator2.default)(_regenerator.default.mark(function _callee3() {
  return _regenerator.default.wrap(function _callee3$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          (0, _react.render)((0, _jsxRuntime.jsx)(ProviderFlow, {
            session: _mocks.mockSession
          }));
          expect(fetchSpy).not.toHaveBeenCalled();
          changeTabVisibility("hidden");
          expect(fetchSpy).not.toHaveBeenCalled();
          changeTabVisibility("visible");
          return _context3.abrupt("return", (0, _react.waitFor)(function () {
            expect(fetchSpy).toHaveBeenCalledTimes(1);
            expect(fetchSpy).toHaveBeenCalledWith("/api/auth/session", expect.anything());
          }));

        case 6:
        case "end":
          return _context3.stop();
      }
    }
  }, _callee3);
})));
test("will refetch the session if told to do so programmatically from another window", (0, _asyncToGenerator2.default)(_regenerator.default.mark(function _callee4() {
  return _regenerator.default.wrap(function _callee4$(_context4) {
    while (1) {
      switch (_context4.prev = _context4.next) {
        case 0:
          (0, _react.render)((0, _jsxRuntime.jsx)(ProviderFlow, {
            session: _mocks.mockSession
          }));
          expect(fetchSpy).not.toHaveBeenCalled();
          changeTabVisibility("hidden");
          expect(fetchSpy).not.toHaveBeenCalled();
          (0, _react2.signOut)({
            redirect: false
          });
          return _context4.abrupt("return", (0, _react.waitFor)(function () {
            expect(fetchSpy).toHaveBeenCalledWith("/api/auth/session", expect.anything());
            expect((0, _utils.printFetchCalls)(fetchSpy.mock.calls)).toMatchInlineSnapshot("\n      Array [\n        \"GET /api/auth/csrf\",\n        \"POST /api/auth/signout\",\n        \"GET /api/auth/session\",\n      ]\n    ");
          }));

        case 6:
        case "end":
          return _context4.stop();
      }
    }
  }, _callee4);
})));
test("allows to customize how often the session will be re-fetched through polling", function () {
  jest.useFakeTimers();
  (0, _react.render)((0, _jsxRuntime.jsx)(ProviderFlow, {
    session: _mocks.mockSession,
    refetchInterval: 1
  }));
  expect(fetchSpy).not.toHaveBeenCalled();
  jest.advanceTimersByTime(1000);
  expect(fetchSpy).toHaveBeenCalledTimes(1);
  expect(fetchSpy).toHaveBeenCalledWith("/api/auth/session", expect.anything());
  jest.advanceTimersByTime(1000);
  expect(fetchSpy).toHaveBeenCalledTimes(2);
  expect((0, _utils.printFetchCalls)(fetchSpy.mock.calls)).toMatchInlineSnapshot("\n      Array [\n        \"GET /api/auth/session\",\n        \"GET /api/auth/session\",\n      ]\n    ");
});
test("allows to customize the URL for session fetching", (0, _asyncToGenerator2.default)(_regenerator.default.mark(function _callee5() {
  var myPath;
  return _regenerator.default.wrap(function _callee5$(_context5) {
    while (1) {
      switch (_context5.prev = _context5.next) {
        case 0:
          myPath = "/api/v1/auth";

          _mocks.server.use(_msw.rest.get("".concat(myPath, "/session"), function (req, res, ctx) {
            return res(ctx.status(200), ctx.json(_mocks.mockSession));
          }));

          (0, _react.render)((0, _jsxRuntime.jsx)(ProviderFlow, {
            session: _mocks.mockSession,
            basePath: myPath
          }));
          expect(fetchSpy).not.toHaveBeenCalled();
          (0, _react2.getSession)();
          return _context5.abrupt("return", (0, _react.waitFor)(function () {
            expect(fetchSpy).toHaveBeenCalledTimes(1);
            expect(fetchSpy).toHaveBeenCalledWith("".concat(myPath, "/session"), expect.anything());
          }));

        case 6:
        case "end":
          return _context5.stop();
      }
    }
  }, _callee5);
})));

function ProviderFlow(props) {
  return (0, _jsxRuntime.jsxs)(_react2.SessionProvider, _objectSpread(_objectSpread({}, props), {}, {
    children: [(0, _jsxRuntime.jsx)(SessionConsumer, {}), (0, _jsxRuntime.jsx)(SessionConsumer, {
      testId: "2"
    })]
  }));
}

function SessionConsumer(_ref6) {
  var _ref6$testId = _ref6.testId,
      testId = _ref6$testId === void 0 ? 1 : _ref6$testId,
      rest = (0, _objectWithoutProperties2.default)(_ref6, _excluded);

  var _useSession = (0, _react2.useSession)(rest),
      session = _useSession.data,
      status = _useSession.status;

  return (0, _jsxRuntime.jsx)("div", {
    "data-testid": "session-".concat(testId),
    children: status === "loading" ? "loading" : JSON.stringify(session)
  });
}

function changeTabVisibility(status) {
  var visibleStates = ["visible", "hidden"];
  if (!visibleStates.includes(status)) return;
  Object.defineProperty(document, "visibilityState", {
    configurable: true,
    value: status
  });
  document.dispatchEvent(new Event("visibilitychange"));
}