"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _msw = require("msw");

var _reactHooks = require("@testing-library/react-hooks");

var _react = require("@testing-library/react");

var _react2 = require("../react");

var _mocks = require("./helpers/mocks");

var _jsxRuntime = require("react/jsx-runtime");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { (0, _defineProperty2.default)(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var origConsoleError = console.error;
var origLocation = window.location;
var locationReplace = jest.fn();
beforeAll(function () {
  console.error = jest.fn();
  delete window.location;
  window.location = _objectSpread(_objectSpread({}, origLocation), {}, {
    replace: locationReplace
  });

  _mocks.server.listen();
});
afterEach(function () {
  _mocks.server.resetHandlers();

  locationReplace.mockClear();
  (0, _react2.signOut)({
    redirect: false
  });
});
afterAll(function () {
  console.error = origConsoleError;
  window.location = origLocation;

  _mocks.server.close();
});
test("it won't allow to fetch the session in isolation without a session context", function () {
  function App() {
    (0, _react2.useSession)();
    return null;
  }

  expect(function () {
    return (0, _react.render)((0, _jsxRuntime.jsx)(App, {}));
  }).toThrow("[next-auth]: `useSession` must be wrapped in a <SessionProvider />");
});
test("when fetching the session, there won't be `data` and `status` will be 'loading'", function () {
  var _renderHook = (0, _reactHooks.renderHook)(function () {
    return (0, _react2.useSession)();
  }, {
    wrapper: _react2.SessionProvider
  }),
      result = _renderHook.result;

  expect(result.current.data).toBe(undefined);
  expect(result.current.status).toBe("loading");
});
test("when session is fetched, `data` will contain the session data and `status` will be 'authenticated'", (0, _asyncToGenerator2.default)(_regenerator.default.mark(function _callee() {
  var _renderHook2, result;

  return _regenerator.default.wrap(function _callee$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _renderHook2 = (0, _reactHooks.renderHook)(function () {
            return (0, _react2.useSession)();
          }, {
            wrapper: _react2.SessionProvider
          }), result = _renderHook2.result;
          _context.next = 3;
          return (0, _react.waitFor)(function () {
            expect(result.current.data).toEqual(_mocks.mockSession);
            expect(result.current.status).toBe("authenticated");
          });

        case 3:
        case "end":
          return _context.stop();
      }
    }
  }, _callee);
})));
test("when it fails to fetch the session, `data` will be null and `status` will be 'unauthenticated'", (0, _asyncToGenerator2.default)(_regenerator.default.mark(function _callee2() {
  var _renderHook3, result;

  return _regenerator.default.wrap(function _callee2$(_context2) {
    while (1) {
      switch (_context2.prev = _context2.next) {
        case 0:
          _mocks.server.use(_msw.rest.get("/api/auth/session", function (req, res, ctx) {
            return res(ctx.status(404), ctx.json(null));
          }));

          _renderHook3 = (0, _reactHooks.renderHook)(function () {
            return (0, _react2.useSession)();
          }, {
            wrapper: _react2.SessionProvider
          }), result = _renderHook3.result;
          return _context2.abrupt("return", (0, _react.waitFor)(function () {
            expect(result.current.data).toEqual(null);
            expect(result.current.status).toBe("unauthenticated");
          }));

        case 3:
        case "end":
          return _context2.stop();
      }
    }
  }, _callee2);
})));
test("it'll redirect to sign-in page if the session is required and the user is not authenticated", (0, _asyncToGenerator2.default)(_regenerator.default.mark(function _callee3() {
  var _renderHook4, result;

  return _regenerator.default.wrap(function _callee3$(_context3) {
    while (1) {
      switch (_context3.prev = _context3.next) {
        case 0:
          _mocks.server.use(_msw.rest.get("/api/auth/session", function (req, res, ctx) {
            return res(ctx.status(404), ctx.json(null));
          }));

          _renderHook4 = (0, _reactHooks.renderHook)(function () {
            return (0, _react2.useSession)({
              required: true
            });
          }, {
            wrapper: _react2.SessionProvider
          }), result = _renderHook4.result;
          _context3.next = 4;
          return (0, _react.waitFor)(function () {
            expect(result.current.data).toEqual(null);
            expect(result.current.status).toBe("unauthenticated");
          });

        case 4:
          expect(locationReplace).toHaveBeenCalledTimes(1);
          expect(locationReplace).toHaveBeenCalledWith(expect.stringContaining("/api/auth/signin"));
          expect(locationReplace).toHaveBeenCalledWith(expect.stringContaining(new URLSearchParams({
            error: "SessionRequired",
            callbackUrl: window.location.href
          }).toString()));

        case 7:
        case "end":
          return _context3.stop();
      }
    }
  }, _callee3);
})));
test("will call custom redirect logic if supplied when the user could not authenticate", (0, _asyncToGenerator2.default)(_regenerator.default.mark(function _callee4() {
  var customRedirect, _renderHook5, result;

  return _regenerator.default.wrap(function _callee4$(_context4) {
    while (1) {
      switch (_context4.prev = _context4.next) {
        case 0:
          _mocks.server.use(_msw.rest.get("/api/auth/session", function (req, res, ctx) {
            return res(ctx.status(404), ctx.json(null));
          }));

          customRedirect = jest.fn();
          _renderHook5 = (0, _reactHooks.renderHook)(function () {
            return (0, _react2.useSession)({
              required: true,
              onUnauthenticated: customRedirect
            });
          }, {
            wrapper: _react2.SessionProvider
          }), result = _renderHook5.result;
          _context4.next = 5;
          return (0, _react.waitFor)(function () {
            expect(result.current.data).toEqual(null);
            expect(result.current.status).toBe("unauthenticated");
          });

        case 5:
          expect(locationReplace).not.toHaveBeenCalled();
          expect(customRedirect).toHaveBeenCalledTimes(1);

        case 7:
        case "end":
          return _context4.stop();
      }
    }
  }, _callee4);
})));